﻿using Kivii;
using Kivii.Lims;
using Kivii.Lims.Entities;
using Kivii.Lims.Extensions;
using Kivii.Linq;
using Kivii.Text;
using Njhg.Third.Semir.Extensions;
using Njhg.Third.Semir.Transforms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Njhg.Third.Semir.Entities
{
    public class SemirOrderBase : IEntityIsThirdReportOrder
    {
        private JsonServiceClient _client = null;

        public Report OnInstanceThirdReport(Guid ThirdReportOrderKvid)
        {
            //return null;
            if (ThirdReportOrderKvid == Guid.Empty) return null;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return null;
            if (third.ThirdType != typeof(SemirOrder).FullName) return null;
            if (third.Metadata == null) return null;
            var semir = third.Metadata.DicToObject();

            var rtns = new Report();
            rtns.BizId = third.SerialNumber;
            rtns.BizKvid = third.Kvid;
            rtns.BizType = typeof(ThirdReportOrder).FullName;
            rtns.ReportId = $"{semir.BRAND_DISPLAY}{DateTime.Now:yyyyMMddHHmmss}";
            //contract.OrderID = semir.DOC_NO;
            //report.Type = "B";
            rtns.TypeEx = "W";
            rtns.TypeName = "委托送样";
            rtns.Category = "SE";
            rtns.DemanderName = semir.COMPANY_NAME;
            rtns.DemanderContactName = semir.CONTACT_PERSON;
            rtns.DemanderContactNumber = semir.CONTACT_PHONE;
            rtns.DemanderContactAddress = semir.COMPANY_ADDRESS;

            rtns.SampleManufacturerName = semir.APPLY_ORG_NAME;
            rtns.SampleManufacturerContactName = semir.PAYER_CONTACT_PERSON;
            rtns.SampleManufacturerContactNumber = semir.PAYER_CONTACT_PHONE;
            rtns.SampleManufacturerAddress = semir.PAYER_ORG_ADDRESS;

            rtns.SampleName = semir.MAT_NAME;
            rtns.SampleBrand = semir.BRAND_DISPLAY;
            rtns.SampleGradeSafety = semir.CHECK_LEVEL;
            rtns.SampleGrade = semir.SAMPLE_GRADE;

            rtns.SampleBatchNumber = semir.FG_MAT_CODE;
            rtns.SampleModel = semir.FG_F2_DISPLAY;

            rtns.Metadata = new Dictionary<string, string>();
            rtns.Metadata["ProductColor"] = semir.FG_COLOR;
            rtns.Metadata["MaterialColor"] = semir.COLOR;
            rtns.Metadata["MaterialNumber"] = semir.MAT_CODE;
            rtns.Metadata["SampleType"] = semir.THIRDPARTY_TESTING_TESTPART;
            rtns.Metadata["VersionNumber"] = semir.MAT_DEV_CODE;
            rtns.Metadata["YearQuarter"] = semir.YEAR+"/"+semir.SEASONS;

            rtns.Metadata["FinishedProductNumber"] = semir.CHECK_STANDARD;
            rtns.Metadata["BatchNumber"] = semir.BATCH_CODE;
            rtns.Remark = semir.REMARK;

            var details = new List<string>();
            foreach (var detail in semir.ITEMS)
            {
                details.Add($"{detail.ITEM_NAME}({detail.ITEM_STANDARD})");
            }
            var tasks = string.Join("、", details.ToArray());
            rtns.Metadata.Add("ERemark", tasks);
            return rtns;
        }

        public bool OnRejectThirdReport(Guid ThirdReportOrderKvid)
        {
            return false;
        }
        public bool OnUploadThirdReport(Guid ThirdReportOrderKvid)
        {
            if (ThirdReportOrderKvid == Guid.Empty) return false;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return false;
            if (third.ThirdType != typeof(SemirOrder).FullName) return false;
            //订单还未受理的 则自动去找相关报告进行关联
            if (!third.IsAccepted) return false;// third.AutoLinkReports();
            if (third.IsUploaded) return true;
            var thirdDetails = conn.Select<ThirdReportOrderDetail>(o => o.ThirdOrderKvid == third.Kvid);
            if (thirdDetails.IsNullOrEmpty()) return false;
            var reports = conn.SelectByIds<Report>(thirdDetails.ConvertAll(o => o.ReportKvid));
            if (reports.IsNullOrEmpty()) return false;
            if (reports.Exists(o => o.Status <= (int)ReportStatusType.ReportIssue)) return false;
            if (reports.Exists(o => o.AmountTest + o.Amount > 0 && o.AmountSettlement <= 0)) return false;
            var result_info = new ReportResultInfo();
            result_info.DOC_NO = third.SerialNumber;
            result_info.URL_PDF = new List<UrlPdf>();

            var rpt_detail = new List<ReportDetail>();

            //针对多个报告存在不合格的项目 则回传订单整体判定为不合格
            var infoJudement = true;
            foreach (var report in reports)
            {
                //if (thirdDetail.ReportKvid == Guid.Empty) continue;
                //var report = reports.FirstOrDefault(o => o.Kvid == thirdDetail.ReportKvid);
                //if (report == null) continue;
                if (report.Status < (int)ReportStatusType.ReportIssue) continue;
                var queryReportItems = conn.From<ReportItem>();
                queryReportItems.Where(o => o.ReportKvid == report.Kvid);
                var allReportItems = conn.Select(queryReportItems);
                var primaryAllReportItems = allReportItems.filterReportItem();
                primaryAllReportItems.ForEach(o => o.PackageUnit = string.Empty);
                var selfItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf).ToList();
                var partItems = allReportItems.Where(o => o.Type == ReportItemType.SamplePart).ToList();
                //合并色号和部位
                if (!selfItems.IsNullOrEmpty())
                {
                    foreach (var item in selfItems)
                    {
                        var primarys = primaryAllReportItems.Where(o => o.RootKvid == item.RootKvid).ToList();
                        foreach (var primary in primarys)
                        {
                            primary.PackageUnit = string.Empty;
                            primary.PackageUnit = item.Title;
                        }
                    }
                }
                if (!partItems.IsNullOrEmpty())
                {
                    foreach (var item in partItems)
                    {
                        var primarys = primaryAllReportItems.Where(o => o.ParentKvid == item.Kvid).ToList();
                        foreach (var primary in primarys)
                        {
                            primary.TitlePart = string.Empty;
                            primary.TitlePart = item.Title;
                        }
                    }
                }

                #region result_info
                result_info.EXT_DOC_NO = report.ReportId;
                result_info.CHECK_FEE += report.AmountSettlement;
                result_info.RESULT = 4;
                if (report.TestJudgement == "合格") result_info.RESULT = 1;
                if (report.TestJudgement == "不合格")
                {
                    infoJudement = false;
                    result_info.RESULT = 2;
                }
                result_info.RESULT_TEXT = report.TestJudgement;
                if (!report.TestNeedJudge) result_info.RESULT_TEXT = "此报告无需判定";

                var code = report.Kvid.ToString().Substring(0, 4);
                var internalCode = $"{code}/{report.ReportId}";

                var urlPdf = new UrlPdf();
                urlPdf.RPT_NO = report.ReportId;
                urlPdf.URL = $"{Configs._pdfUrl}/Lims/Report/{internalCode}";
                urlPdf.RESULT = 4;
                if (report.TestJudgement == "合格") urlPdf.RESULT = 1;
                if (report.TestJudgement == "不合格") urlPdf.RESULT = 2;
                urlPdf.RPT_DESC = report.TestJudgement;
                if (!report.TestNeedJudge) urlPdf.RPT_DESC = "此报告无需判定";
                result_info.URL_PDF.Add(urlPdf);
                #endregion

                #region rpt_detail
                var reportDetail = new ReportDetail();
                reportDetail.RPT_NO = report.ReportId;
                reportDetail.URL = $"{Configs._pdfUrl}/Lims/Report/{internalCode}";
                reportDetail.RESULT = 4;
                if (report.TestJudgement == "合格") reportDetail.RESULT = 1;
                if (report.TestJudgement == "不合格") reportDetail.RESULT = 2;
                reportDetail.RPT_DESC = report.TestJudgement;
                if (!report.TestNeedJudge) reportDetail.RPT_DESC = "此报告无需判定";

                reportDetail.ITEMS = new List<Detail>();
                reportDetail.ITEMS = primaryAllReportItems.ConverToDetails(allReportItems);
                rpt_detail.Add(reportDetail);
                #endregion
            }

            //针对多个报告存在不合格的项目 则回传订单整体判定为不合格
            if (!infoJudement)
            {
                result_info.RESULT = 2;
                result_info.RESULT_TEXT = "不合格";
            }
            if (_client == null) _client = new JsonServiceClient(Configs._defaultUrl);

            var info = JsonSerializer.SerializeToString(result_info);
            var rpt = JsonSerializer.SerializeToString(rpt_detail);

            var body = new Dictionary<string, string>();
            body.Add("result_info", info);
            body.Add("rpt_detail", rpt);

            var resp = _client.Post<SemirOrderResponse>($"{Configs.RouteUpdateWtsResult}?token={Configs._token}", body);
            Console.WriteLine($"Code:{resp.code}\n");
            Console.WriteLine($"Message:{resp.message}\n");
            Console.WriteLine($"resp_code:{resp.data.resp_code}\n");
            Console.WriteLine($"data.message:{resp.data.message}\n");
            Console.WriteLine($"data.errors:{resp.data.errors}\n");
            Console.WriteLine($"recordsTotal:{resp.data.recordsTotal}\n");
            Console.WriteLine($"results:{resp.data.results}\n");
            if (resp.code == 200 || resp.code == 404)
            {
                third.IsUploaded = true;
                third.AddOnlyProperties(o => o.IsUploaded);
                conn.UpdateOnly(third);

                var log = new EntityLog<ThirdReportOrder>();
                log.OwnerKvid = third.Kvid;
                log.BizId = third.SerialNumber;
                log.BizKvid = third.Kvid;
                log.BizType = third.ThirdType;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Remark = $"Code:{resp.code}  Message:{resp.message}";
                conn.Insert(log);
                if (resp.code == 200)
                {
                    try
                    {
                        var bodyClose = new Dictionary<string, string>();
                        bodyClose.Add("DOC_NO", result_info.DOC_NO);
                        _client.Post<SemirOrderResponse>($"{Configs.RouteCloseWts}?token={Configs._token}", bodyClose);
                    }
                    catch { }

                }
                return true;
            }
            else return false;
        }

        public object OnPullThirdReport()
        {
            #region 设置缓存1分钟内不能连续请求
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = KiviiContext.GetUrnKey("SemirOrderAccept:LockedOneMinute");
            var cacheValue = cache.Get<string>(cacheKey);
            if (!cacheValue.IsNullOrEmpty())
            {
                throw new Exception(cacheValue);
            }
            #endregion

            var rtns = new RestfulQueryResponse<ThirdReportOrder>();

            if (_client == null) _client = new JsonServiceClient(Configs._defaultUrl);

            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime), p => p.ThirdType == typeof(SemirOrder).FullName);
            //DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime));

            var response = _client.PullSemirOrder(Configs._token, maxTranslateTime);
            cache.Set(cacheKey, "一分钟内不可多次查询!", TimeSpan.FromMinutes(1));

            (response.code != 200).ThrowIfTrue(response.message);
            rtns.Results = new List<ThirdReportOrder>();
            if (response.data == null) return rtns;
            if (response.data.results.IsNullOrEmpty()) return rtns;

            var results = response.data.results.AcceptThirdContractOrder(Guid.Parse(Configs._ownerKvid), Configs._ownerName, conn);
            rtns.Results.AddRange(results);
            rtns.Total = rtns.Results.Count;

            var thread = KiviiContext.NewThread(() =>
            {
                //没有关联报告的 自动执行符合条件的关联
                SemirExtension.LinkReports(null, _client, Configs._token);
            });
            thread.Start();
            return rtns;
        }
    }
}
