﻿using Kivii;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Text;
using Njhg.Third.Semir.Entities;
using Njhg.Third.Semir.Transforms;
using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Njhg.Third.Semir.Extensions
{
    public static class SemirExtension
    {
        /// <summary>
        /// 拉取森马订单
        /// </summary>
        /// <param name="baseUrl"></param>
        /// <param name="token"></param>
        /// <returns></returns>
        internal static SemirOrderResponse PullSemirOrder(this JsonServiceClient _client, string token, DateTime? beginTime = null, DateTime? endTime = null, int length = 200)
        {
            (_client == null).ThrowIfTrue("PullSemirOrder:请传入_client");
            (token.IsNullOrEmpty()).ThrowIfTrue("PullSemirOrder:请传入token");
            //var client = new JsonServiceClient(baseUrl);

            var getWtsListRequest = new SemirOrderGetWtsList();
            getWtsListRequest.token = token;
            getWtsListRequest.length = length;
            if (beginTime != null) getWtsListRequest.begin_time = beginTime.Value;
            if (endTime != null) getWtsListRequest.end_time = endTime.Value;

            var response = _client.Get(getWtsListRequest);
            return response;
        }

        /// <summary>
        /// 创建第三方订单数据
        /// </summary>
        /// <param name="semirOrders"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        internal static List<ThirdReportOrder> AcceptThirdContractOrder(this List<SemirOrder> semirOrders, Guid? ownerKvid = null, string ownerName = null, IDbConnection conn = null)
        {
            (semirOrders.IsNullOrEmpty()).ThrowIfTrue("AcceptThirdContractOrder:请传入semirOrders");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            }

            var rtns = new List<ThirdReportOrder>();

            //找出最小创建日期的订单
            var minCreateTime = semirOrders.Min(o => o.CREATED_TIME);
            if (minCreateTime == DateTime.MinValue) minCreateTime = DateTime.Now;
            var queryThirdReportOrders = conn.From<ThirdReportOrder>();
            queryThirdReportOrders.Where(o => o.OperateTime >= minCreateTime & o.ThirdType == typeof(SemirOrder).FullName);
            queryThirdReportOrders.Select(o => o.SerialNumber);
            var thirdReportOrders = conn.Column<string>(queryThirdReportOrders);

            //IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            //if (useTransaction) trans = conn.OpenTransaction();

            try
            {
                foreach (var item in semirOrders)
                {
                    if (thirdReportOrders.Exists(o => o == item.DOC_NO)) continue;

                    var third = new ThirdReportOrder();
                    try
                    {

                        if (ownerKvid != null) third.OwnerKvid = ownerKvid.Value;
                        if (!ownerName.IsNullOrEmpty()) third.OwnerName = ownerName;
                        third.SerialNumber = item.DOC_NO.RemoveControlCharacters();
                        third.Name = item.MAT_NAME.Length > 100 ? item.MAT_NAME.Substring(0, 99) : item.MAT_NAME.RemoveControlCharacters(); ;
                        third.ThirdName = item.BRAND_DISPLAY.RemoveControlCharacters(); ;
                        third.ThirdType = typeof(SemirOrder).FullName.RemoveControlCharacters(); ;
                        third.IsAccepted = false;
                        third.IsUploaded = false;
                        third.OperateTime = item.CREATED_TIME;

                        third.ManufacturerName = item.APPLY_ORG_NAME.RemoveControlCharacters(); ;
                        third.ManufacturerContactName = item.PAYER_CONTACT_PERSON.RemoveControlCharacters(); ;
                        third.SampleName = item.MAT_NAME.Length > 100 ? item.MAT_NAME.Substring(0, 99) : item.MAT_NAME.RemoveControlCharacters(); ;
                        third.BatchNumber = item.FG_MAT_CODE.RemoveControlCharacters(); ;
                        third.Brand = item.BRAND_DISPLAY.RemoveControlCharacters(); ;
                        //thirdReportOrder.Content = JsonSerializer.SerializeToString(item);
                    }
                    catch (Exception ex)
                    {

                        throw ex;
                    }
                    third.Metadata = new Dictionary<string, string>();
                    foreach (var propertyInfo in item.GetType().GetProperties())
                    {
                        try
                        {
                            if (propertyInfo.Name == "ITEMS")
                            {
                                var items = item.ITEMS;
                                if (items.IsNullOrEmpty()) items = new List<SemirOrderDetail>();
                                third.Metadata.Add(propertyInfo.Name, JsonSerializer.SerializeToString(items).RemoveControlCharacters());
                                continue;
                            }
                            var value = propertyInfo.GetValue(item);
                            third.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString().RemoveControlCharacters());
                        }
                        catch (Exception ex)
                        {

                            throw ex;
                        }
                    }

                    try
                    {
                        conn.Insert(third);
                        //trans?.Commit();
                        third.RemoveAllOnlyProperties();
                        rtns.Add(third);
                    }
                    catch (Exception ex)
                    {
                        throw ex;
                    }


                }
            }
            catch (Exception ex)
            {
                //trans?.Rollback();
                throw ex;
            }

            return rtns;
        }

        internal static bool ConfirmAccept(this JsonServiceClient _client, string token, ThirdReportOrder thirdReportOrder)
        {
            (_client == null).ThrowIfTrue("ConfirmAccept:请传入_client");
            (token.IsNullOrEmpty()).ThrowIfTrue("ConfirmAccept:请传入token");
            var body = new Dictionary<string, string>();
            body.Add("DOC_NO", thirdReportOrder.SerialNumber);
            var response = _client.Post<SemirOrderResponse>($"{Configs.RouteAcceptWts}?token={token}", body);
            if (response.code != 200) return false;
            return true;
        }

        /// <summary>
        /// 自动关联报告,规则:存在不合格的报告不自动关联,只有全合格的才自动关联
        /// </summary>
        /// <param name="third"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        internal static void AutoLinkReports(this ThirdReportOrder third, IDbConnection conn = null)
        {
            third.ThrowIfNull("请传入订单信息!");
            (third.Kvid == Guid.Empty).ThrowIfTrue("请传入订单Kvid信息!");
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();

            //third = conn.SingleById<ThirdContractOrder>(third.Kvid);
            //third.ThrowIfNull("未找到对应订单信息!");
            if (third.IsAccepted) return;

            var queryReports = conn.From<Report>();
            queryReports.Where(o => o.BizId == third.SerialNumber & o.Status >= (int)ReportStatusType.ReportIssue).OrderBy(o => o.ReportId);
            var reports = conn.Select(queryReports);
            if (reports.IsNullOrEmpty()) return;

            var infoJudement = true;
            if (reports.Exists(o => o.TestJudgement == "不合格")) infoJudement = false;

            if (!infoJudement)
            {
                third.ReportIds = "存在不合格报告请手动关联!";
                third.AddOnlyProperties(o => o.ReportIds);
                conn.UpdateOnly(third);
                return;
            }
            //var trans = conn.OpenTransaction();
            try
            {
                third.ReportIds = string.Empty;
                var reportIds = new List<string>();
                foreach (var report in reports)
                {
                    report.BizId = third.SerialNumber;
                    report.AddOnlyProperties(o => o.BizId);
                    report.BizKvid = third.Kvid;
                    report.AddOnlyProperties(o => o.BizKvid);
                    report.BizType = typeof(ThirdReportOrder).FullName;
                    report.AddOnlyProperties(o => o.BizType);
                    conn.UpdateOnly(report);

                    var thirdDetail = new ThirdReportOrderDetail();
                    thirdDetail.ThirdOrderKvid = third.Kvid;
                    thirdDetail.ReportId = report.ReportId;
                    thirdDetail.ReportKvid = report.Kvid;
                    thirdDetail.ThirdType = third.ThirdType;
                    thirdDetail.SerialNumber = third.SerialNumber;
                    thirdDetail.Name = third.Name;
                    thirdDetail.ThirdName = third.ThirdName;
                    thirdDetail.IsUploaded = false;
                    thirdDetail.OperateTime = DateTime.Now;
                    thirdDetail.Metadata = new Dictionary<string, string>();
                    thirdDetail.Metadata = report.Metadata;
                    foreach (var propertyInfo in report.GetType().GetProperties())
                    {
                        if (propertyInfo.Name == "Metadata") continue;
                        try
                        {
                            var value = propertyInfo.GetValue(report);
                            thirdDetail.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                        }
                        catch { }

                    }
                    conn.Insert(thirdDetail);
                    reportIds.Add(report.ReportId);
                }

                if (third.Metadata.IsNullOrEmpty()) third.Metadata = new Dictionary<string, string>();
                third.Metadata["Amount"] = reports.Sum(o => o.Amount + o.AmountTest).ToString();
                third.AddOnlyProperties(o => o.Metadata);
                third.ReportIds = string.Join("、", reportIds);
                third.AddOnlyProperties(o => o.ReportIds);
                third.IsAccepted = true;
                third.AddOnlyProperties(o => o.IsAccepted);
                conn.UpdateOnly(third);

                //trans.Commit();
            }
            catch (Exception ex)
            {
                //trans.Rollback();
                throw ex;
            }
        }

        internal static void LinkReports(List<ThirdReportOrder> thirds = null, JsonServiceClient _client = null, string token = null)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            if (thirds.IsNullOrEmpty())
            {
                thirds = conn.Select<ThirdReportOrder>(o => o.IsAccepted == false && o.IsOffseted == false && o.IsUploaded == false && o.ThirdType == typeof(SemirOrder).FullName);
            }
            if (thirds.IsNullOrEmpty()) return;
            var queryReports = conn.From<Report>();
            queryReports.Where(o => Sql.In(o.BizId, thirds.ConvertAll(p => p.SerialNumber))).OrderBy(o => o.ReportId);
            var allReports = conn.Select(queryReports);
            if (allReports.IsNullOrEmpty()) return;

            foreach (var third in thirds)
            {
                var reports = allReports.Where(o => o.BizId == third.SerialNumber).ToList();
                if (reports.IsNullOrEmpty()) continue;
                if (reports.Exists(o => o.TestJudgement == "不合格"))
                {
                    third.ReportIds = "存在不合格报告请手动关联!";
                    third.AddOnlyProperties(o => o.ReportIds);
                    conn.UpdateOnly(third);
                    continue;
                }
                third.ReportIds = string.Empty;
                var reportIds = new List<string>();
                foreach (var report in reports)
                {
                    report.BizId = third.SerialNumber;
                    report.AddOnlyProperties(o => o.BizId);
                    report.BizKvid = third.Kvid;
                    report.AddOnlyProperties(o => o.BizKvid);
                    report.BizType = typeof(ThirdReportOrder).FullName;
                    report.AddOnlyProperties(o => o.BizType);
                    conn.UpdateOnly(report);

                    var thirdDetail = new ThirdReportOrderDetail();
                    thirdDetail.ThirdOrderKvid = third.Kvid;
                    thirdDetail.ReportId = report.ReportId;
                    thirdDetail.ReportKvid = report.Kvid;
                    thirdDetail.ThirdType = third.ThirdType;
                    thirdDetail.SerialNumber = third.SerialNumber;
                    thirdDetail.Name = third.Name;
                    thirdDetail.ThirdName = third.ThirdName;
                    thirdDetail.IsUploaded = false;
                    thirdDetail.OperateTime = DateTime.Now;
                    thirdDetail.Metadata = new Dictionary<string, string>();
                    thirdDetail.Metadata = report.Metadata;
                    foreach (var propertyInfo in report.GetType().GetProperties())
                    {
                        if (propertyInfo.Name == "Metadata") continue;
                        try
                        {
                            var value = propertyInfo.GetValue(report);
                            thirdDetail.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                        }
                        catch { }

                    }
                    conn.Insert(thirdDetail);
                    reportIds.Add(report.ReportId);
                }

                if (third.Metadata.IsNullOrEmpty()) third.Metadata = new Dictionary<string, string>();
                third.Metadata["Amount"] = reports.Sum(o => o.Amount + o.AmountTest).ToString();
                third.AddOnlyProperties(o => o.Metadata);
                third.ReportIds = string.Join("、", reportIds);
                third.AddOnlyProperties(o => o.ReportIds);
                third.IsAccepted = true;
                third.AddOnlyProperties(o => o.IsAccepted);
                conn.UpdateOnly(third);
                if (_client != null && token != null) _client.ConfirmAccept(token, third);
            }
        }

        /// <summary>
        /// 字典类型转化为对象
        /// </summary>
        /// <param name="dic"></param>
        /// <returns></returns>
        public static SemirOrder DicToObject(this Dictionary<string, string> dic)
        {
            var md = new SemirOrder();
            CultureInfo cultureInfo = Thread.CurrentThread.CurrentCulture;
            TextInfo textInfo = cultureInfo.TextInfo;
            foreach (var d in dic)
            {
                var filed = textInfo.ToTitleCase(d.Key);
                try
                {
                    var value = d.Value;
                    var property = md.GetType().GetProperty(filed);
                    if (property == null) continue;
                    if (!property.PropertyType.IsGenericType)
                    {
                        switch (property.Name)
                        {
                            case "ITEMS":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<List<SemirOrderDetail>>(value), null);
                                break;
                            default:
                                property.SetValue(md, string.IsNullOrEmpty(value) ? null : Convert.ChangeType(value, property.PropertyType), null);
                                break;
                        }
                    }
                    else
                    {
                        Type genericTypeDefinition = property.PropertyType.GetGenericTypeDefinition();
                        switch (property.Name)
                        {
                            case "ITEMS":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<List<SemirOrderDetail>>(value), null);
                                break;
                            default:
                                //泛型Nullable<>
                                if (genericTypeDefinition == typeof(Nullable<>))
                                {
                                    property.SetValue(md, string.IsNullOrEmpty(value) ? null : Convert.ChangeType(value, Nullable.GetUnderlyingType(property.PropertyType)), null);
                                }
                                break;
                        }
                    }
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
            return md;
        }

        /// <summary>
        /// 获取森马关系数据（带缓存，缓存10天）
        /// </summary>
        /// <param name="forceRefresh">是否强制刷新缓存</param>
        /// <returns>森马关系数据列表</returns>
        internal static List<SemirRelation> GetCachedSemirRelations(bool forceRefresh = false)
        {
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = KiviiContext.GetUrnKey("SemirRelationQuery:Cache10Days");

            if (!forceRefresh)
            {
                var cacheValue = cache.Get<List<SemirRelation>>(cacheKey);
                if (!cacheValue.IsNullOrEmpty())
                {
                    return cacheValue;
                }
            }

            // 缓存中没有数据或强制刷新，从数据库查询
            var conn = KiviiContext.GetOpenedDbConnection<SemirRelation>();
            var results = conn.Select<SemirRelation>();

            // 将结果缓存10天
            cache.Set(cacheKey, results, TimeSpan.FromDays(10));

            return results;
        }

        public static List<Detail> ConverToDetails(this List<ReportItem> reportItems, List<ReportItem> allReportItems)
        {
            var rtns = new List<Detail>();
            var exists = GetCachedSemirRelations();
            foreach (var item in reportItems)
            {
                var nodeItems = allReportItems.Where(o => o.ParentKvid == item.Kvid).ToList();

                #region 拼接检测要求及检测值
                var spec2List = new List<string>();
                var resultList = new List<string>();
                if (item.Type == ReportItemType.Test)
                {
                    spec2List.Add($"{item.Title}({item.DetectionValueQualified})");
                    resultList.Add($"{item.Title}({item.TestValue})");
                }
                else if (!nodeItems.IsNullOrEmpty())
                {
                    foreach (var node in nodeItems)
                    {
                        spec2List.Add($"{node.Title}({node.DetectionValueQualified})");
                        resultList.Add($"{node.Title}({node.TestValue})");
                    }
                }
                var spec2 = string.Join(", ", spec2List.ToArray());
                var result = string.Join(", ", resultList.ToArray());
                #endregion

                var detail = new Detail();
                detail.ITEM_CODE = "H-9999";
                detail.ITEM_STANDARD_CODE = "F-9999";
                if (!exists.IsNullOrEmpty())
                {
                    var title = item.Title.Replace(" ", "");
                    var code = item.DetectionCodeActual.Replace(" ", "").Replace("—", "-");
                    var existTitle = exists.FirstOrDefault(o => o.Title.Replace(" ", "").Contains(title) || title.Contains(o.Title.Replace(" ", "")));
                    if (existTitle != null)
                    {
                        detail.ITEM_CODE = existTitle.Code;
                    }
                    var existCode = exists.FirstOrDefault(o => o.Title.Replace(" ", "").Contains(code) || code.Contains(o.Title.Replace(" ", "")));
                    if (existCode != null)
                    {
                        detail.ITEM_STANDARD_CODE = existCode.Code;
                    }
                }
                detail.ITEM_NAME = item.Title;
                detail.ITEM_DESC = item.Title;
                detail.TEMP_ITEM_ID = "";
                detail.F1_DISPLAY = item.PackageUnit;
                detail.ITEM_STANDARD = item.DetectionCodeActual;
                detail.ITEM_SPEC1 = "";
                detail.ITEM_SPEC2 = spec2;
                detail.CHECK_PART = item.TitlePart;
                detail.CHECK_PART_DISPLAY = item.TitlePart;
                detail.RESULT = 4;
                if (item.Judgement == "合格") detail.RESULT = 1;
                else if (item.Judgement == "不合格")
                {
                    detail.RESULT = 0;
                }
                else detail.RESULT_TEXT = "此项无需判定";
                detail.RESULT_REMARK = result;
                rtns.Add(detail);
            }

            return rtns;
        }

        /// <summary>
        /// 移除字符串中的所有不可打印的 Unicode 控制字符
        /// </summary>
        /// <param name="input">原始字符串</param>
        /// <returns>移除控制字符后的字符串</returns>
        public static string RemoveControlCharacters(this string input)
        {
            if (string.IsNullOrEmpty(input))
                return input;

            // 使用 LINQ 过滤掉控制字符
            return new string(input.Where(c => !char.IsControl(c)).ToArray());
        }
    }
}
