﻿using Kivii;
using Kivii.Linq;
using Kivii.Office.OpenXml.Excel;
using Kivii.Web;
using Njhg.Third.Semir.Entities;
using Njhg.Third.Semir.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Njhg.Third.Semir.Transforms
{
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class SemirRelationCreate : RestfulCreate<SemirRelation>
    {
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class SemirRelationUpdate : RestfulUpdate<SemirRelation>
    {
    }
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class SemirRelationRead : RestfulRead<SemirRelation>
    {
    }
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class SemirRelationQuery : RestfulQuery<SemirRelation>
    {
    }
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class SemirRelationDelete : RestfulDelete<SemirRelation>
    {
    }

    public class SemirRelationImportResponse : RestfulQueryResponse<SemirRelation>
    {
        public bool success { get; set; }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class SemirRelationImport : RestfulExecution<SemirRelation>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Request.Files.Count() < 1 || Request.Files == null).ThrowIfTrue("未找到excel文件!");
            var httpFile = Request.Files[0];
            ExcelPackage package = new ExcelPackage(Request.Files[0].InputStream);
            (package.Workbook.Worksheets.Count == 0).ThrowIfTrue("未找到Sheet!");

            var conn = KiviiContext.GetOpenedDbConnection<SemirRelation>();

            var rtns = new SemirRelationImportResponse();
            rtns.Results = new List<SemirRelation>();
            var exists = SemirExtension.GetCachedSemirRelations();
            try
            {
                var sheet = package.Workbook.Worksheets[1];
                var end = sheet.Dimension.End.Row;

                var results = new List<SemirRelation>();

                for (int i = 2; i <= end; i++)
                {
                    var title1 = sheet.Cells[i, 4].Value;
                    var code1 = sheet.Cells[i, 5].Value;
                    var title2 = sheet.Cells[i, 6].Value;
                    var code2 = sheet.Cells[i, 7].Value;

                    if (title1 != null && code1 != null)
                    {

                        var reltn1 = new SemirRelation();
                        reltn1.Title = title1.ToString().Trim();
                        reltn1.Code = code1.ToString().Trim();
                        if (!results.Exists(o => o.Title == reltn1.Title && o.Code == reltn1.Code)) results.Add(reltn1);
                        //conn.Insert(reltn1);
                        //rtns.Results.Add(reltn1);

                    }

                    if (title2 != null && code2 != null)
                    {
                        var reltn2 = new SemirRelation();
                        reltn2.Title = title2.ToString().Trim();
                        reltn2.Code = code2.ToString().Trim();
                        if (!results.Exists(o => o.Title == reltn2.Title && o.Code == reltn2.Code)) results.Add(reltn2);
                        //conn.Insert(reltn2);
                        //rtns.Results.Add(reltn2);

                    }
                }

                if (!results.IsNullOrEmpty())
                {
                    var preInserts = new List<SemirRelation>();
                    foreach (var item in results)
                    {
                        var exist = false;
                        if (!exists.IsNullOrEmpty())
                        {
                            if (exists.Exists(o => o.Title == item.Title && o.Code == item.Code)) exist = true;
                        }
                        if (!exist)
                        {
                            preInserts.Add(item);
                        }
                    }

                    if (!preInserts.IsNullOrEmpty())
                    {
                        conn.InsertAll(preInserts);
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                SemirExtension.GetCachedSemirRelations(true);
            }

            rtns.success = true;
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
}
